%Please refer to the parameters of inference in this paper for more details: 
%Distributed Message Passing for Large Scale Graphical Models(CVPR 2011)

%DATASET_NAME = 'msrc';

if ~exist('DATASET_NAME','var') || isempty(DATASET_NAME)
    DATASET_NAME = 'msrc';
end
DATASET_NAME = lower(DATASET_NAME);
ALLPATHS.DATASET_NAME = DATASET_NAME;

CODE_MPIDIR_PATH = '/usr/lib64/openmpi/bin'; %you have to change the path to openmpi
CODE_HRCF_PATH = 'external/libHCRF';
CODE_DCBP_PATH = 'external/cBP-MPIGeneral';
CODE_VOCEVAL_PATH = 'external/VOCdevkit';
CODE_UTILITY_PATH = 'utility';
CODE_FEATURE_PATH = 'computeFeature';

mpath = matlabpath;
if ~numel(strfind(lower(mpath), lower(CODE_MPIDIR_PATH))), addpath(CODE_MPIDIR_PATH); end;
if ~numel(strfind(lower(mpath), lower(CODE_HRCF_PATH))), addpath(CODE_HRCF_PATH); end;
if ~numel(strfind(lower(mpath), lower(CODE_DCBP_PATH))), addpath(CODE_DCBP_PATH); end;
if ~numel(strfind(lower(mpath), lower(CODE_VOCEVAL_PATH))), addpath(CODE_VOCEVAL_PATH); end;
if ~numel(strfind(lower(mpath), lower(CODE_UTILITY_PATH))), addpath(CODE_UTILITY_PATH); end;
if ~numel(strfind(lower(mpath), lower(CODE_FEATURE_PATH))), addpath(CODE_FEATURE_PATH); end;

% PASCAL dataset
VOC_TRAIN_FILE = 'data/VOC2010/Train.txt';
VOC_TEST_FILE = 'data/VOC2010/Test.txt';

VOC_ROOT = fullfile('data/VOC2010');
VOC_PATH = fullfile(VOC_ROOT, 'Images');
VOC_LEARNING_PATH = fullfile(VOC_ROOT,'Learning');
VOC_RESULTS_PATH = fullfile(VOC_ROOT,'Results','segFile');

% MSRC dataset
MSRC_TRAIN_FILE = 'data/MSRC/Train.txt';
MSRC_TEST_FILE = 'data/MSRC/Test.txt';

MSRC_ROOT = fullfile('data/MSRC');
MSRC_PATH = fullfile(MSRC_ROOT,'Images');
MSRC_LEARNING_PATH = fullfile(MSRC_ROOT,'Learning');
MSRC_RESULTS_PATH = fullfile(MSRC_ROOT,'Results','segFile');

%K1 and K2 are two parameters to select the superpixels and super segments
%from Gpb. The higher K is, the less superpixels in the image

%NUM_STATES: the number of classes each superpixel can take

%IGNORE_CLASSES: the classes that we will not consider in our segmentation,
%Like in MSRC-21, we ignore the classes: Mountain and Horse

%DET_CLASSES: choose how many classes of detection we are gonna use
%in the segmentation, in default for VOC, we use all the detection bounding
%box(background is class 1). For MSRC, we will use all the 16 classes where
%DET_CLASSES = [5 7 10 12 13 14 15 16 17 18 19 21 22 23 24]';
%You can either input class indices directly (by setting ind_classes) like
%what we do now. Or you can also search for the class name like
%{'car','cat'...} 

if exist('DATASET_NAME', 'var')
    switch DATASET_NAME
              
        case 'msrc'
            NUM_STATES = 21;
            IGNORE_CLASSES = [1,6,9]';  % MSRC-21 which ingores void, horse and mountain
            %DET_CLASSES = [5 7 10 12 13 14 15 16 17 18 19 21 22 23 24]';%{'cow', 'sheep', 'aeroplane', 'face', 'car','bicycle', 'flower', 'sign', 'bird', 'book', 'chair', 'cat', 'dog', 'body', 'boat'};
            DATASET_ROOT = MSRC_ROOT;
            IMAGES_PATH = MSRC_PATH; 
	    TRAIN_FILE = MSRC_TRAIN_FILE;
	    TEST_FILE = MSRC_TEST_FILE;
            LEARNING_PATH = MSRC_LEARNING_PATH;
            RESULTS_PATH = MSRC_RESULTS_PATH;
            if ~exist('K1', 'var'), K1 = 0.08; end;
            if ~exist('K2', 'var'), K2 = 0.12; end;
            if ~exist('BBOX_NAME', 'var'), BBOX_NAME ='lsvm-16classes'; end;
            
        case 'voc'
            NUM_STATES = 21;
            IGNORE_CLASSES = [];
            %DET_CLASSES = (2:21)';
            DATASET_ROOT = VOC_ROOT;
            IMAGES_PATH = VOC_PATH;
	    TRAIN_FILE = VOC_TRAIN_FILE;
	    TEST_FILE = VOC_TEST_FILE;
            LEARNING_PATH = VOC_LEARNING_PATH;
            RESULTS_PATH = VOC_RESULTS_PATH;
            if ~exist('K1', 'var'), K1 = 0.14; end;
            if ~exist('K2', 'var'), K2 = 0.18; end;
            if ~exist('BBOX_NAME', 'var'), BBOX_NAME ='LSVM'; end;
    end
end

ALLPATHS.DATA_PATH = DATASET_ROOT;
ALLPATHS.IMAGES_PATH = IMAGES_PATH;
ALLPATHS.LEARNING_PATH = LEARNING_PATH;
ALLPATHS.RESULTS_PATH = RESULTS_PATH;
ALLPATHS.TRAIN_FILE = TRAIN_FILE;
ALLPATHS.TEST_FILE = TEST_FILE; 
ALLPATHS.PIXEL_PATH = fullfile(ALLPATHS.DATA_PATH, 'Features', sprintf('SuperSegmentPotential%0.2f', K1));
ALLPATHS.SUPIXEL_PATH = fullfile(ALLPATHS.DATA_PATH, 'Features', sprintf('SuperSegmentPotential%0.2f', K2));
ALLPATHS.BBOX_PATH = fullfile(ALLPATHS.DATA_PATH, 'Features', BBOX_NAME);
ALLPATHS.PN_PATH = fullfile(ALLPATHS.DATA_PATH , 'Features', sprintf('PnEdges%0.2f-%0.2f',K1, K2));
ALLPATHS.Z_FEATURE_PATH = fullfile(ALLPATHS.DATA_PATH,'Features','Zfeature');
ALLPATHS.CHOWLIU_PATH = fullfile(ALLPATHS.Z_FEATURE_PATH,'ChowLiu');
ALLPATHS.GT_PATH = fullfile(ALLPATHS.DATA_PATH , 'GroundTruth');
ALLPATHS.UCM1_PATH = fullfile(ALLPATHS.DATA_PATH , 'SuperSegments', sprintf('SuperSegment%0.2f', K1));
ALLPATHS.UCM2_PATH = fullfile(ALLPATHS.DATA_PATH , 'SuperSegments', sprintf('SuperSegment%0.2f', K2));
ALLPATHS.SCENE_PATH = fullfile(ALLPATHS.DATA_PATH, 'Features','Scenes');
ALLPATHS.IGNORE_CLASSES = IGNORE_CLASSES;
ALLPATHS.NUM_STATES = NUM_STATES;



