function [acc average globalAccuracy] = pixelwiseAccuracy(gtPATH,rPATH, supPATH, modelname, DATASET_NAME, featparams, ind_images)

if nargin < 5, DATASET_NAME = 'msrc'; end;
dataset_globals;

[classes, gtcols] = getclassinfo(ALLPATHS.DATASET_NAME);
if numel(IGNORE_CLASSES)
   classes = filterclasses(classes, IGNORE_CLASSES);   % classes
   gtcols(IGNORE_CLASSES,:) = [];
end;
nclasses = length(classes);

fid = fopen(fullfile(ALLPATHS.DATA_PATH, 'Test.txt'),'r');
T = textscan(fid,'%s');
test = T{1};
if isempty(ind_images)
    NumSamples = length(test);
else
    NumSamples = length(ind_images);
end
outpath = fullfile(ALLPATHS.RESULTS_PATH, '..', 'accuracy');
if ~exist(outpath, 'dir'), mkdir(outpath); end;
accuracy = zeros(nclasses,NumSamples);
groundTruth = zeros(nclasses,NumSamples);

results_dir = fullfile(rPATH, 'ReImages');
if ~exist(results_dir,'dir')
    mkdir(results_dir);
end

for imgk = 1:NumSamples
    if ~isempty(ind_images)
        x = ind_images(imgk);
    else
        x = imgk;
    end
    [~, name, ~] = fileparts(test{x});
    %groundtruth
    labelfile = fullfile(gtPATH,[name '_labels.png']);
    if exist(labelfile, 'file')
        gtim = imread(labelfile);
    else
       GT = imread(fullfile(gtPATH,[name '_GT.bmp']));    
       gtim = get_GTclasslabels(GT, gtcols);
       gtim = uint8(gtim);
       imwrite(gtim, labelfile);
    end;

    % Testing
    if max(gtim(:)) > nclasses
        fprintf('Error! \n');
        return;
    end

    % results file
    segresultsfile = fullfile(rPATH, [name '.mat']);
    superpixelfile = fullfile(supPATH, [name '_label.mat']);
    resdata = load(segresultsfile);
    if ~featparams.segOnly
       ressup = resdata.RPixel';
    else
       ressup = resdata.RSeg';
    end;
    supdata = load(superpixelfile);
    resim = getsegim(ressup, supdata.labels, size(gtim));

    switch(DATASET_NAME)
        case 'voc'
            outName = fullfile(results_dir,[name '.png']);
            if ~exist(outName,'file')
                imwrite(resim,VOClabelcolormap,outName,'png');
            end
            continue;
            
        case 'msrc'
            resim(gtim == 0) = 0;
            unClasses = unique(gtim);
            unClasses(unClasses==0) = [];

            for i = 1:numel(unClasses)
                Ind_G = gtim == unClasses(i);
                Ind_R = resim == unClasses(i);
                groundTruth(unClasses(i),x) = nnz(Ind_G);
                temp = Ind_G + Ind_R;
                Corr = nnz(temp > 1);

                if Corr == 0
                    accuracy(unClasses(i),x) = 1e-5;
                else
                    accuracy(unClasses(i),x) = Corr;
                end
            end        
    end
end

if strcmp(DATASET_NAME,'msrc')
    acc = zeros(nclasses,1);
    for i = 1:nclasses
        [~, ~, v] = find(accuracy(i,:));
        [~, ~, v_] = find(groundTruth(i,:));
        acc(i) = sum(v)/sum(v_);
    end
    acc(sum(groundTruth,2) == 0) = [];
    average = mean(acc);
    globalAccuracy = floor(sum(accuracy(:)))/sum(groundTruth(:));
    save(fullfile(outpath, ['segperformance_' modelname '.mat']), 'classes', 'acc', 'average', 'globalAccuracy');
    saveperformance(fullfile(outpath, ['segperformance_' modelname '.txt']), modelname, acc, average, globalAccuracy, classes)
end

if strcmp(DATASET_NAME,'voc')
    VOCopts = SegInit(results_dir);
    [accuracies,avacc,~] = VOCevalseg(VOCopts,[]);
    saveperformance(fullfile(outpath, ['segperformance_' modelname '.txt']), modelname, accuracies, avacc, [], classes)
end

height=480;
htop = 40;
hf = figure('position',[200,200,660,height]);
subplot('position',[0,1-(htop-8)/height,1,(htop-8)/height]);
c = [1, 0.5, 0.6];
topim(:, :, 1) = c(1) * ones(10, 100);
topim(:, :, 2) = c(2) * ones(10, 100);
topim(:, :, 3) = c(3) * ones(10, 100);
imagesc(topim);
axis off;
text(18,5, sprintf('MSRC accuracy: average (%0.3f), global (%0.3f)', average, globalAccuracy), 'fontsize', 15)
axis off;
subplot('position',[0.045,0.01,0.63,1-htop/height-0.004-0.01]);
h = bar(acc);
set(gca, 'xlim', [0.5,21.5])
bar_child = get(h,'Children');
set(bar_child,'facevertexcdata',gtcols/255);
subplot('position',[0.72,0,0.05,1-htop/height])
plot_colorbar(classes, gtcols/255, 21*10)
subplot('position',[0.79,0,0.21,1-htop/height])
plot_class_labels(classes, acc);
outfile = fullfile(outpath, ['segperformance_' modelname '.png']);
%export_fig(outfile);
set(gcf, 'color', [1,1,1])
%F = getframe(gcf);
%imwrite(F.cdata, outfile);
try
    export_fig(outfile); 
end;
close(hf);

function  plot_colorbar(classes, gtcols, imsize)

h = round(imsize(1)/length(classes));
w = round(h*1.2);
collegend = zeros(h*length(classes), w, 3);

for j=1:length(classes)
    for k = 1:3
       collegend((j-1)*h+1:j*h, 1:w, k) = gtcols(j, k);
    end;
end;
imagesc(uint8(collegend*255))
axis off;


function  plot_class_labels(classes, acc)

for j=1:length(classes)
    hold on;
    col = [0,0,0];
    text(0.015,1-(j-1)/length(classes)-0.024, [classes(j).class, sprintf(' (%0.3f)', acc(j))], 'fontsize', 11.5, 'color', col, 'fontweight', 'bold');
end;
axis off;  


function segim = getsegim(ressup, supim, imsize)


    segim = zeros(imsize);
    for i = 1 : size(ressup, 1)
        [~, cls] = max(ressup(i, :), [], 2);
        ind = find(supim == i);
        if numel(ind)
            segim(ind) = cls;
        end;
    end;
    
function saveperformance(outfile, modelname, acc, average, globalAccuracy, classes)


    fid = fopen(outfile, 'w+');
    fprintf(fid, '%s\n', modelname);
    fprintf(fid, '------------\n');
    fprintf(fid, 'ACCURACY PER CLASS\n'); 
    for i=1:21, fprintf(fid, '%0.4f   %s\n', acc(i),classes(i).class); end; 
    fprintf(fid, '------------\n');
    fprintf(fid, 'AVERAGE\n');
    fprintf(fid, '%0.4f\n', average);
    fprintf(fid, '------------\n');
    fprintf(fid, 'GLOBAL\n');  
    fprintf(fid, '%0.4f\n', globalAccuracy);
    fclose(fid);

    fprintf('%s\n', modelname);
    fprintf('------------\n');
    fprintf('ACCURACY PER CLASS\n'); 
    for i=1:21, fprintf('%0.4f   %s\n', acc(i),classes(i).class); end; 
    fprintf('------------\n');
    fprintf('AVERAGE\n');
    fprintf('%0.4f\n', average);
    fprintf('------------\n');
    fprintf('GLOBAL\n');  
    fprintf('%0.4f\n', globalAccuracy);
    fprintf('\n');
    fprintf('Results written out to: %s\n', outfile);








